
#include <windows.h>  
#include <commctrl.h>
#include "header.h"  


#if defined (WIN32)
	#define IS_WIN32 TRUE
#else
	#define IS_WIN32 FALSE
#endif

#define IS_NT      IS_WIN32 && (BOOL)(GetVersion() < 0x80000000)
#define IS_WIN32S  IS_WIN32 && (BOOL)(!(IS_NT) && (LOBYTE(LOWORD(GetVersion()))<4))
#define IS_WIN95   (BOOL)(!(IS_NT) && !(IS_WIN32S)) && IS_WIN32

HINSTANCE hInst;   // current instance

LPCTSTR lpszAppName = "MyApp";
LPCTSTR lpszTitle   = "Header Control"; 


BOOL RegisterWin95( CONST WNDCLASS* lpwc );


int APIENTRY WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance,
                      LPTSTR lpCmdLine, int nCmdShow)
{
   MSG      msg;
   HWND     hWnd; 
   WNDCLASS wc;

   wc.style         = CS_HREDRAW | CS_VREDRAW;
   wc.lpfnWndProc   = (WNDPROC)WndProc;       
   wc.cbClsExtra    = 0;                      
   wc.cbWndExtra    = 0;                      
   wc.hInstance     = hInstance;              
   wc.hIcon         = LoadIcon (hInstance, lpszAppName); 
   wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
   wc.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
   wc.lpszMenuName  = lpszAppName;              
   wc.lpszClassName = lpszAppName;              

   if ( IS_WIN95 )
   {
      if ( !RegisterWin95( &wc ) )
         return( FALSE );
   }
   else if ( !RegisterClass( &wc ) )
      return( FALSE );

   hInst = hInstance; 

   hWnd = CreateWindow( lpszAppName, 
                        lpszTitle,    
                        WS_OVERLAPPEDWINDOW, 
                        CW_USEDEFAULT, 0, 
                        CW_USEDEFAULT, 0,  
                        NULL,              
                        NULL,              
                        hInstance,         
                        NULL               
                      );

   if ( !hWnd ) 
      return( FALSE );

   ShowWindow( hWnd, nCmdShow ); 
   UpdateWindow( hWnd );         

   while( GetMessage( &msg, NULL, 0, 0) )   
   {
      TranslateMessage( &msg ); 
      DispatchMessage( &msg );  
   }

   return( msg.wParam ); 
}


BOOL RegisterWin95( CONST WNDCLASS* lpwc )
{
   WNDCLASSEX wcex;

   wcex.style         = lpwc->style;
   wcex.lpfnWndProc   = lpwc->lpfnWndProc;
   wcex.cbClsExtra    = lpwc->cbClsExtra;
   wcex.cbWndExtra    = lpwc->cbWndExtra;
   wcex.hInstance     = lpwc->hInstance;
   wcex.hIcon         = lpwc->hIcon;
   wcex.hCursor       = lpwc->hCursor;
   wcex.hbrBackground = lpwc->hbrBackground;
   wcex.lpszMenuName  = lpwc->lpszMenuName;
   wcex.lpszClassName = lpwc->lpszClassName;

   // Added elements for Windows 95.
   //...............................
   wcex.cbSize = sizeof(WNDCLASSEX);
   wcex.hIconSm = LoadImage(wcex.hInstance, lpwc->lpszClassName, 
                            IMAGE_ICON, 16, 16,
                            LR_DEFAULTCOLOR );
			
   return RegisterClassEx( &wcex );
}

LPCTSTR pszTitles[] = { "Name", "Type", "Size" };

int GetColumnOffs( HWND hHeader, int nColNum )
{
   HD_ITEM hi;
   int     i;
   int     x = 0;

   hi.mask = HDI_WIDTH;

   // Calculate the column position.
   //...............................
   for( i=0; i<nColNum; i++ )
   {
      Header_GetItem( hHeader, i, &hi );
      x += hi.cxy;
   }

   return( x );
}

LRESULT CALLBACK WndProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
static HD_LAYOUT hl;
static WINDOWPOS wp;
static RECT      rc;
static HDC       hdc;
static int       nCurItem;
static HWND      hHeader = NULL;

   switch( uMsg )
   {
      case WM_CREATE :
              InitCommonControls();

              // Create the header control.
              //...........................
              hHeader = CreateWindowEx( 0, WC_HEADER, "",
                                        WS_CHILD | WS_VISIBLE |
                                        HDS_BUTTONS,
                                        0, 0, 10, 10, hWnd, 
                                        (HMENU)1, hInst, NULL );

              // Initialize the HD_LAYOUT members.
              //..................................
              hl.pwpos = &wp;
              hl.prc   = &rc;

              if ( hHeader )
              {
                 HD_ITEM hi;
                 int     i;

                 SendMessage( hHeader, WM_SETFONT, 
                              (WPARAM)GetStockObject( DEFAULT_GUI_FONT ), 0 );

                 for( i=0; i<3; i++ )
                 {
                    hi.mask    = HDI_FORMAT | HDI_TEXT | HDI_WIDTH;
                    hi.pszText = (LPTSTR)pszTitles[i];
                    hi.fmt     = HDF_STRING;
                    hi.cxy     = 100;

                    Header_InsertItem( hHeader, i, (LPARAM)&hi ); 
                 }
              }
              break;

      case WM_SIZE :
              rc.top    = 0;
              rc.left   = 0;
              rc.right  = LOWORD( lParam );
              rc.bottom = HIWORD( lParam );

              Header_Layout( hHeader, &hl );
              SetWindowPos( hHeader, hWnd, wp.x, wp.y, wp.cx, wp.cy, wp.flags ); 
              break;

      case WM_CONTEXTMENU :
              {
                 HD_HITTESTINFO hi;

                 hi.pt.x = LOWORD( lParam );
                 hi.pt.y = HIWORD( lParam );

                 // Convert the coordinates to client.
                 //...................................
                 ScreenToClient( hHeader, &hi.pt );

                 // Test for a hit.
                 //................
                 SendMessage( hHeader, HDM_HITTEST, 0, (LPARAM)&hi );

                 // If a hit occurred, display menu.
                 //.................................                 
                 if ( hi.flags & HHT_ONHEADER )
                 {
                    HD_ITEM hdi;
                    HMENU   hMenu = CreatePopupMenu();

                    // Retrieve the current format.
                    //.............................
                    hdi.mask = HDI_FORMAT;
                    Header_GetItem( hHeader, hi.iItem, &hdi );

                    nCurItem = hi.iItem;

                    // Build the menu and display it.
                    //...............................
                    AppendMenu( hMenu, MF_STRING, IDM_LEFT, "&Left" );
                    AppendMenu( hMenu, MF_STRING, IDM_CENTER, "&Center" );
                    AppendMenu( hMenu, MF_STRING, IDM_RIGHT, "&Right" );

                    CheckMenuRadioItem( hMenu, IDM_LEFT, IDM_RIGHT,
                           (hdi.fmt & HDF_JUSTIFYMASK) == HDF_LEFT ? IDM_LEFT :
                           (hdi.fmt & HDF_JUSTIFYMASK) == HDF_CENTER ? IDM_CENTER :
                           IDM_RIGHT, MF_BYCOMMAND );

                    TrackPopupMenu( hMenu, TPM_LEFTALIGN | TPM_LEFTBUTTON |
                                              TPM_RIGHTBUTTON,
                                              LOWORD( lParam ), HIWORD( lParam ),
                                              0, hWnd, NULL );
                    DestroyMenu( hMenu );
                 }
              }
              break;

      case WM_NOTIFY :
              {
                 int        x;
                 HD_NOTIFY* pHN = (HD_NOTIFY*)lParam;

                 switch( pHN->hdr.code )
                 {
                    case HDN_BEGINTRACK :
                         // Only allow Left Mouse Drag.
                         //............................
                         if ( pHN->iButton != 0 )
                            return( TRUE );

                         // Get the column offset.
                         //.......................
                         x = GetColumnOffs( hHeader, pHN->iItem ) +
                                pHN->pitem->cxy;

                         GetClientRect( hWnd, &rc );

                         // Fix up rect for the invert.
                         //............................
                         rc.left   = x;
                         rc.top    = wp.cy;
                         rc.right  = x+1;

                         // Get the window DC and invert the rect.
                         //.......................................
                         hdc = GetDC( hWnd );
                         InvertRect( hdc, &rc );

                         break;

                    case HDN_TRACK :
                         // Turn off the invert on the old rect.
                         //.....................................
                         InvertRect( hdc, &rc );

                         // Get the column offset.
                         //.......................
                         x = GetColumnOffs( hHeader, pHN->iItem ) +
                                pHN->pitem->cxy;

                         GetClientRect( hWnd, &rc );

                         // Invert the new rect.
                         //.....................
                         rc.left   = x;
                         rc.top    = wp.cy;
                         rc.right  = x+1;
                         InvertRect( hdc, &rc );
                         break;

                    case HDN_ENDTRACK :
                         // Turn off the invert and release the DC.
                         //........................................
                         InvertRect( hdc, &rc );
                         ReleaseDC( hWnd, hdc );
                         break;

                    case HDN_ITEMCHANGING :
                         // Do not allow the first column to be changed.
                         //.............................................
                         if ( pHN->iItem == 0 )
                            return( TRUE );

                         break;

                    case HDN_ITEMCHANGED :
                         // Update the contents of the window to reflect
                         // the new column alignment.
                         //.............................................
                         break;

                    case HDN_ITEMCLICK :
                         // Sort the column's contents
                         //...........................
                         break;
                 }
              }
              break;

      case WM_COMMAND :
              switch( LOWORD( wParam ) )
              {
                 case IDM_LEFT :
                 case IDM_RIGHT :
                 case IDM_CENTER :
                        {
                           HD_ITEM hdi;

                           // Update the alignment of the column.
                           //....................................
                           hdi.mask = HDI_FORMAT;
                           hdi.fmt  = HDF_STRING + 
                                ( LOWORD( wParam ) == IDM_LEFT ? HDF_LEFT :
                                  LOWORD( wParam ) == IDM_CENTER ? HDF_CENTER :
                                  HDF_RIGHT );

                           Header_SetItem( hHeader, nCurItem, &hdi );
                        }
                        break;

                 case IDM_ABOUT :
                        DialogBox( hInst, "AboutBox", hWnd, (DLGPROC)About );
                        break;

                 case IDM_EXIT :
                        DestroyWindow( hWnd );
                        break;
              }
              break;
      
      case WM_DESTROY :
              PostQuitMessage(0);
              break;

      default :
            return( DefWindowProc( hWnd, uMsg, wParam, lParam ) );
   }

   return( 0L );
}


LRESULT CALLBACK About( HWND hDlg,           
                        UINT message,        
                        WPARAM wParam,       
                        LPARAM lParam)
{
   switch (message) 
   {
       case WM_INITDIALOG: 
               return (TRUE);

       case WM_COMMAND:                              
               if (   LOWORD(wParam) == IDOK         
                   || LOWORD(wParam) == IDCANCEL)    
               {
                       EndDialog(hDlg, TRUE);        
                       return (TRUE);
               }
               break;
   }

   return (FALSE); 
}
